/*
* Launcher.java - DynamicBees, a demo for the probe and watcher mechanisms
* Copyright (C) 1998-2002 Olivier Gutknecht, Fabien Michel
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//package dynamicbees;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import madkit.kernel.AbstractAgent;
import madkit.kernel.Kernel;

/**
  @version 2.0
  @author Fabien MICHEL 01/02/2001*/
public class Launcher extends madkit.kernel.Agent
{
    // requis pour la sérialisation (!?!), 3* -> envirt
	private static final long serialVersionUID = 33L;

	int delay = 9;
	int robotnumber = 5;
	int height=900, width=1200;
	List<AbstractAgent> agentsList = new ArrayList<AbstractAgent>();
	List<Robot> robotsList = new ArrayList<Robot>();
	boolean active=false, started=false;
	boolean withSwing=false;
	boolean randomMode=true;
	Scheduler robotScheduler;
	Environment robotEnvironment;
	View robotViewer;
	
	public Launcher(){}
	
	public void setWidth (int add){width = add; robotEnvironment.xMax = add;}
	public int getWidth(){return width;}
	public void setHeight (int add){height = add; robotEnvironment.yMax = add;} 
	public int getHeight(){return height;}
	public int getDelay() {return delay;}
	public void setDelay(int  v)
	{
		this.delay = v;
		if (robotScheduler!=null)
			robotScheduler.delay=delay;
	}
	public void setRobotnumber(int  v) {this.robotnumber = v;}
	public int getRobotnumber() {return robotnumber;}
	public boolean getActive() {return active;}
	public void setActive(boolean  v) {this.active = v;}
	public boolean getwithSwing() {return withSwing;}
	public void setwithSwing(boolean  v) {this.withSwing = v;}
	public Environment environment() { return robotEnvironment; }
	
	public void activate()
	{
		println("Launching simulation...");
		createGroup(true,"react","robots",null,null);
		requestRole("react","robots","launcher",null);
		active = true;
	}

	public void start()
	{
		if (active) {	
			robotScheduler=new Scheduler(delay);
			agentsList.add(robotScheduler);
			robotEnvironment=new Environment(width, height, -1);
			agentsList.add(robotEnvironment);
			robotViewer=new View(withSwing, width, height);
			agentsList.add(robotViewer);
			launchRobots(1, true);  // launch a leader
			// launch remaining robots (followers)
			launchRobots(robotnumber - 1, false);  
			
			//println("launching time : "+(System.currentTimeMillis()-timeL));
			launchAgent(robotEnvironment,"robot world",false);
			launchAgent(robotViewer,"viewer",true);
			launchAgent(robotScheduler,"robot scheduler",false);
			started = true;
			println("activated");
		}
	}

	public void live()
	{
		while(true)
		{
			exitImmediatlyOnKill();
			pause(500 + (int) (Math.random()*2000) );
	    }
		
	} 

	public void end()
	{
		if (active && started) {
			println("simulation ending");
			killAgent(robotViewer);  
			killAgent(robotScheduler);
			killAgent(robotEnvironment);
			pause(200);
			//long timeL = System.currentTimeMillis();
			for (Iterator<AbstractAgent> i =agentsList.iterator(); i.hasNext();)
			{
				AbstractAgent a = (AbstractAgent) i.next();
				killAgent(a);
			}
			for (Iterator<Robot> i = robotsList.iterator(); i.hasNext();)
			{
				Robot r = (Robot) i.next();
				//if ((++j)%1000==0) System.out.println(""+j+" robots killed");
				killAgent(r);
			}
			started=false;
			//System.err.println("killing time : "+(System.currentTimeMillis()-timeL));
			pause(1000);
			Kernel.debugString();
			System.gc();
		}
	}
    
	public void initGUI()
	{
		setGUIObject(new LauncherGUI(this));
	}

	public synchronized void launchRobots(int numberOfRobots, boolean isLeader)
	{
		int id = robotsList.size();
		for(int nb = 0; nb < numberOfRobots; nb++) {
			Robot robot = new Robot(id, isLeader);
			launchAgent(robot, isLeader ? "leader" : "robot", false);
			id++;
			robotsList.add(robot);
		}
	}
}	

class LauncherGUI extends JPanel implements ActionListener
{
    // requis pour la sérialisation (!?!), 1* -> GUI
	private static final long serialVersionUID = 13L;
	
	JButton launchSimu,launchLeader,launchRobot,launchViewer,randomMode;
	Launcher myLauncher;
	JSlider g1_sldr, g5_sldr;
	
	LauncherGUI(Launcher myLauncher)
	{
		launchSimu = addNewButton("Launch simulation");
		//randomMode = addNewButton("Random launch Off");
		launchLeader = addNewButton("Launch a new swarm");
		launchRobot = addNewButton("Launch a robot");
		//launchViewer = addNewButton("Launch a viewer");
		this.myLauncher = myLauncher;
		JSlider simulationSpeed = new JSlider(JSlider.HORIZONTAL, 0, 19 , 11);
		simulationSpeed.addChangeListener(new SliderListener());
		simulationSpeed.setMajorTickSpacing(5);
		simulationSpeed.setPaintTicks(true);
		simulationSpeed.setPaintLabels(false);
		simulationSpeed.setBorder(BorderFactory.createEmptyBorder(0,0,15,0));
		add(simulationSpeed);
		setPreferredSize( new java.awt.Dimension(300,220) );
	}
	
	public JButton addNewButton(String name)
	{
		JButton jb = new JButton(name);
		add(jb);
		jb.addActionListener(this);
		return jb;
	}

    public void chooseCoefs() 
    { 
    	final String coeffNames[] = {"Avoidance", "Cohesion", "Alignment"};
		final int nbCoef = coeffNames.length;
		double[] coefs = {1., 1., 1.}; 
		JPanel box = new JPanel();
		javax.swing.JTextField txts[] = new javax.swing.JTextField[nbCoef];
		int i;
		for(i = 0; i < nbCoef; i++) {
		    box.add( new javax.swing.JLabel(coeffNames[i] + " = ") );
		    txts[i] = new javax.swing.JTextField("" + coefs[i]);
		    box.add( txts[i] );
		}
		if (javax.swing.JOptionPane.showConfirmDialog
		     (null, box, "Select the Behaviour Coefficients", 
		      javax.swing.JOptionPane.OK_CANCEL_OPTION, 
		      javax.swing.JOptionPane.QUESTION_MESSAGE) 
		    == javax.swing.JOptionPane.OK_OPTION) {
		    for(i = 0; i < nbCoef; i++)
		    	coefs[i] = Double.parseDouble( txts[i].getText() );
		    Robot.setCoefs(coefs);
		}
    }
	
    public void actionPerformed(ActionEvent e)
    {
    	Object s=e.getSource();
    	if (s==launchSimu) 
    	{ 
    		launchSimu.setEnabled(false); /* the following does not work
    		if ( myLauncher.getActive() )  myLauncher.end();  // pb!!!
    		else { // 
    			launchSimu.setText("Restart simulation");
    			launchSimu.repaint();
    		} */
    		chooseCoefs();  // to choose coefficients at start
    		myLauncher.activate(); 
    		myLauncher.start(); 
    	}
    	else if (s==randomMode)
    	{
    		if (myLauncher.randomMode)
    		{
    			randomMode.setText("Random launch On");
    			myLauncher.randomMode = false;
    		}
    		else
    		{
    			randomMode.setText("Random launch Off");
    			myLauncher.randomMode = true;
    		}
    	}
    	else if (s==launchLeader)
    		myLauncher.launchRobots(1, true);
    	else if (s==launchRobot)
    		myLauncher.launchRobots(1, false);
    	else if(s==launchViewer)
    	{
    		View v = new View(myLauncher.withSwing, myLauncher.width,myLauncher.height);
    		myLauncher.agentsList.add(0,v);
    		myLauncher.launchAgent(v,"viewer",true);
    	}
    }			
	
    class SliderListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
		    JSlider source = (JSlider)e.getSource();
		    if (!source.getValueIsAdjusting()) {
			myLauncher.setDelay( 20 - (int)source.getValue());
		    }
		}
    }
	
    class CoeffChange implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
		}
	}	
} // fin de la classe LauncherGUI
