/*
* View.java - DynamicBees, a demo for the probe and watcher mechanisms
* Copyright (C) 1998-2002 Olivier Gutknecht, Fabien Michel
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//package dynamicbees;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;

/**
  @version 2.0
  @author Fabien MICHEL 01/02/2001*/
public class View extends madkit.kernel.Watcher
    implements madkit.kernel.ReferenceableAgent {
    // requis pour la sérialisation (!?!), 3* -> envirt
    private static final long serialVersionUID = 34L;

    Component onScreen;

    RobotEnumerator robotEnumerator;
  
    int width=300;
    int height=300;
    
    boolean show = true;
    boolean swing = true;
    public void setShow (boolean add){show = add;}
    public boolean getShow(){return show;}
 
    public View()
    {
    }
    public View(boolean b, int w, int h)
    {
    	this();
    	swing = b;
    	width=w;
    	height=h + 20;
    }

    public void initGUI()
    {
    	if (swing)
		setGUIObject(onScreen = new GridCanvas(width,height,this));
	else 
		setGUIObject(onScreen = new AwtGridCanvas(width,height,this));
    }
    
    public void paintRobots(Graphics g) { 
    	final double radius_2 = Robot.radius / 2, pi_3 = Math.PI / 3;
    	final int  iRad = (int)(Robot.radius), iDiam = (int)(Robot.diameter), 
    			iPi_3 = (int)( OrientPoint.rad2deg(pi_3) ), 
    			iDperc = (int)(2 * Robot.Rpercept);
    	Robot[] robots = robotEnumerator.getRobots();
    	if (robots != null)	
    		for(int i = 0; i < robots.length; i++)
    			if (robots[i].color() != null)
	    		{	
    				final OrientPoint config = robots[i].config();
    				final Vector2D    pos = config.position();
					final double orient = config.orientation(); /*
    				System.err.print("--- Drawing "); 
    				System.err.print(robots[i].leader() ? "leader" : "follower"); 
    				System.err.print(" at ("); 
    				System.err.print( (int) pos.getX() );
    				System.err.print(", "); 
    				System.err.print( (int) pos.getY() );
    				System.err.print(", "); 
    				System.err.print( (int)(orient * convRadDeg) );
    				System.err.println(") ---"); */
					g.setColor( robots[i].color() );
					Vector2D  v0  = Vector2D.fromPolar(Robot.radius, orient),
							v1    = v0.rotate(pi_3), v2 = v0.rotate(- pi_3);
					final Vector2D[] pts = { pos.plus(v0), pos.plus(v1), pos.minus(v2), 
							   		   		 pos.minus(v1), pos.plus(v2), pos.plus(v0)};
					int[] x = new int[pts.length], y = new int[pts.length] ;
					for (int j = 0; j < pts.length; j++)
					{ x[j] = (int)( pts[j].getX() ); y[j] = (int)( pts[j].getY() ); }
					if ( robots[i].leader() )  g.fillPolygon(x, y, pts.length);
					else g.fillArc( (int)(pos.getX() - Robot.radius), 
									(int)(pos.getY() - Robot.radius), iDiam - 1, iDiam - 1,
									// angle is inverted, as Y axis is
									(int)( OrientPoint.rad2deg(-orient - 2 * pi_3) ), 
									4 * iPi_3 ); 
					// helps to distinguish leaders from followers
					g.setColor( robots[i].enCollision() ? Color.red : 
								robots[i].neighbours() ? Color.orange : Color.green.darker() );
					if ( robots[i].leader() ) 
						g.fillOval( (int)(pos.getX() - radius_2), 
									(int)(pos.getY() - radius_2), iRad, iRad ); 
					else {
						final Vector2D frontHalf = pos.plus( v0.mult(.5) );
						final int iX = (int) frontHalf.getX(), iY = (int) frontHalf.getY();
						final int[] x3 = {iX, x[2], x[3], iX}, y3 = {iY, y[2], y[3], iY};
						g.fillPolygon(x3, y3, 4);
					}
					final double percRad = Robot.Rpercept / Robot.radius;
					final Vector2D vp1 = v0.rotate(Robot.Apercept), 
							vp2 = v0.rotate(- Robot.Apercept);
					final Vector2D[] ptsPrc = 
						{ pos.plus(vp1), pos.plus( vp1.mult(percRad) ),
						  pos.plus(vp2), pos.plus( vp2.mult(percRad) ) };
					final int[] xp1 = { (int)( ptsPrc[0].getX() ), (int)( ptsPrc[1].getX() ) },
							xp2 =     { (int)( ptsPrc[2].getX() ), (int)( ptsPrc[3].getX() ) },
							yp1 =     { (int)( ptsPrc[0].getY() ), (int)( ptsPrc[1].getY() ) },
							yp2 =     { (int)( ptsPrc[2].getY() ), (int)( ptsPrc[3].getY() ) }; 
					g.drawPolyline(xp1, yp1, 2);
					g.drawArc( (int)(pos.getX() - Robot.Rpercept),
							   (int)(pos.getY() - Robot.Rpercept), iDperc, iDperc, 
							   (int)( OrientPoint.rad2deg(-orient - Robot.Apercept) ), 
							   (int)( OrientPoint.rad2deg(2 *Robot.Apercept) ) );
					g.drawPolyline(xp2, yp2, 2);
			    } 
}

    synchronized public void observe()
    {
	if (show)
		if (swing)
		   onScreen.repaint();
		else
		   ((AwtGridCanvas)onScreen).display();
    }

    public void activate()
    {
    	if (! swing) ((AwtGridCanvas)onScreen).initialisation();
    	createGroup(true,"react","robots",null,null);
        requestRole("react","robots","robot observer",null);
        robotEnumerator=new RobotEnumerator("robots", "robot", null);
        addProbe(robotEnumerator);
        //System.err.println("View activated");
    }
    
    public void end()
    {
    	disposeMyGUI();
    	super.end();
   }
    
}

class AwtGridCanvas extends Canvas
{
    // requis pour la sérialisation (!?!), 1* -> GUI
	private static final long serialVersionUID = 12L;
	
  View simuViewer;
  Image buffer;
  Graphics bufferGraphics;

  public AwtGridCanvas(int width,int height,View l)
    {
    setBackground(Color.black);
    setForeground(Color.blue);
	setSize(width,height);
	simuViewer=l;
    }
  
 void initialisation()
  {
	Dimension d = getSize();
	buffer = createImage(d.width,d.height);
   	bufferGraphics = buffer.getGraphics();
   	bufferGraphics.setColor(Color.black);
	bufferGraphics.fillRect(0,0,d.width,d.height);
  }

  public Dimension getPreferredSize() {return getSize();}

  void display()
  {
	Dimension d = getSize();
   	bufferGraphics.setColor(Color.black);
	bufferGraphics.fillRect(0,0,d.width,d.height);
	simuViewer.paintRobots(bufferGraphics);
	getGraphics().drawImage(buffer,0,0,this);
  }

}
